unit Sdimain;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Menus, Dialogs,
     StdCtrls, Buttons, ExtCtrls, SysUtils, Printers, Search, Messages, ClipBrd,
     ShellAPI;

type
  TWinPadForm = class(TForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    SaveItem: TMenuItem;
    ExitItem: TMenuItem;
    N1: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    HelpMenu: TMenuItem;
    AboutItem: TMenuItem;
    StatusBar: TPanel;
    SpeedPanel: TPanel;
    MyTextFile: TMemo;
    PrintDialog: TPrintDialog;
    EditPopUpMenu: TPopupMenu;
    CutPopUp: TMenuItem;
    CopyPopUp: TMenuItem;
    PastePopUp: TMenuItem;
    DeletePopUp: TMenuItem;
    SelectAllPopUp: TMenuItem;
    EditMenu: TMenuItem;
    OpenItem: TMenuItem;
    NewItem: TMenuItem;
    CutItem: TMenuItem;
    CopyItem: TMenuItem;
    PasteItem: TMenuItem;
    DeleteItem: TMenuItem;
    SelectAllItem: TMenuItem;
    TimeDateItem: TMenuItem;
    SaveAsItem: TMenuItem;
    PrintItem: TMenuItem;
    FindDialog: TFindDialog;
    WordWrapItem: TMenuItem;
    N2: TMenuItem;
    PrinterSetupDialog: TPrinterSetupDialog;
    PrintSetup: TMenuItem;
    SearchMenu: TMenuItem;
    FindItem: TMenuItem;
    FindNextItem: TMenuItem;
    N3: TMenuItem;
    ReplaceDialog: TReplaceDialog;
    ReplaceItem: TMenuItem;
    FontDialog: TFontDialog;
    FontItem: TMenuItem;
    N4: TMenuItem;
    UndoMemo: TMemo;
    UndoItem: TMenuItem;
    N5: TMenuItem;
    UndoPopUp: TMenuItem;
    ExitBtn: TSpeedButton;
    HelpBtn: TSpeedButton;
    UndoBtn: TSpeedButton;
    PasteBtn1: TSpeedButton;
    CopyBtn: TSpeedButton;
    CutBtn: TSpeedButton;
    PrintBtn: TSpeedButton;
    SaveBtn: TSpeedButton;
    OpenBtn: TSpeedButton;
    NewBtn: TSpeedButton;
    FindBtn: TSpeedButton;
    ReplaceBtn: TSpeedButton;
    PrintSetupBtn: TSpeedButton;
    DeleteBtn: TSpeedButton;
    FontBtn: TSpeedButton;
    DateTimeBtn: TSpeedButton;
    N6: TMenuItem;
    N7: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure ExitBtnClick(Sender: TObject);
    procedure OpenBtnClick(Sender: TObject);
    procedure NewBtnClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure PrintBtnClick(Sender: TObject);
    procedure CutBtnClick(Sender: TObject);
    procedure CopyBtnClick(Sender: TObject);
    procedure PasteBtnClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure SelectAllBtnClick(Sender: TObject);
    procedure TimeDateItemClick(Sender: TObject);
    procedure SaveAsItemClick(Sender: TObject);
    procedure FindBtnClick(Sender: TObject);
    procedure WordWrapItemClick(Sender: TObject);
    procedure PrintSetupClick(Sender: TObject);
    procedure ReplaceBtnClick(Sender: TObject);
    procedure Find(Sender: TObject);
    procedure FindNextItemClick(Sender: TObject);
    procedure Replace(Sender: TObject);
    procedure FontItemClick(Sender: TObject);
    procedure UndoBtnClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    FileOpened : Boolean;
    procedure DropFiles(var Msg : TWMDropFiles); message WM_DROPFILES;
    procedure OpenFile;
    procedure SaveForUndo;
  public
    { Public declarations }
    procedure ShowHint(Sender: TObject);
  end;

var
   WinPadForm : TWinPadForm;

implementation

uses About;

{$R *.DFM}

procedure TWinPadForm.SaveForUndo;
{ Save all the information required to 'undo' the previous editing operation }
begin
     with UndoMemo do
     begin
          Clear;
          WordWrap := MyTextFile.WordWrap;
          Lines    := MyTextFile.Lines;
          Font     := MyTextFile.Font;
     end;
end;

procedure TWinPadForm.OpenFile;
{ Open the file specified by the open dialog }
begin
     MyTextFile.Lines.LoadFromFile(OpenDialog.Filename);
     FileOpened := True;
     WinPadForm.Caption := 'WinPad - ' + ansilowercase(OpenDialog.Filename);
     Application.Title := 'WinPad - ' + ansilowercase(OpenDialog.Filename);
end;

procedure TWinPadForm.DropFiles(var Msg : TWMDropFiles);
{ Open any files dropped on the application's form or minimised icon }
var
   Filename           : array [0..255] of Char;
   NrOfFiles, BufSize : Word;

begin
     { If multiple files are dropped , inform the user and exit }
     NrOfFiles := DragQueryFile(Msg.Drop, Word(-1), Filename, BufSize);
     if NrOfFiles > 1 then
     begin
          MessageDlg('Hey! Only one file at a time...',mtInformation, [mbOK],0);
          DragFinish(Msg.Drop);
          Exit;
     end;

     { Get the name of the file that was dropped onto the form }
     DragQueryFile(Msg.Drop, 0, Filename, BufSize);

     if not MyTextFile.Modified and not FileOpened then
     begin
          SaveForUndo;
          OpenDialog.FileName := StrPas(Filename);
          OpenFile;
     end
     else
         if MessageDlg('Close Current File?', mtInformation,[mbOK,mbCancel],0)
                                                                     = mrOK then
         begin
              SaveForUndo;
              OpenDialog.FileName := StrPas(Filename);
              OpenFile;
         end;
end;

procedure TWinPadForm.ShowHint(Sender: TObject);
{ Enable status bar to show application hints from 'on-hint' events }
begin
     StatusBar.Caption := Application.Hint;
end;

procedure TWinPadForm.FormCreate(Sender: TObject);
{ Initialise the application and open any file specified at the command line }
begin
     Application.OnHint    := ShowHint;  { Display application hints in status }
     DragAcceptFiles(WinPadForm.Handle, True);   { We'll accept dropped files }

     { If a file is entered as a command line parameter, open it at startup }
     if ParamCount > 0 then
     begin
          if FileExists(ParamStr(1)) then
          begin
               OpenDialog.Filename := ParamStr(1);
               SaveDialog.Filename := ParamStr(1);
               OpenFile;
               SaveForUndo;
          end
          else
              FileOpened := False;
     end;
end;

procedure TWinPadForm.ExitBtnClick(Sender: TObject);
{ Upon exiting, confirm if the user wants to leave the application }
begin
     if MessageDlg('Really Quit WinPad?',mtInformation,[mbOK,mbCancel],0) = mrOK
                                                                            then
         Close;
end;

procedure TWinPadForm.NewBtnClick(Sender: TObject);
{ If a file is open or has been modified confirm if the user wishes to close
{ the file before beginning a new file. Otherwise, begin a new file }
begin
     if FileOpened or MyTextFile.Modified then
        if MessageDlg('Close Current File?',mtInformation,[mbOK,mbCancel],0)
                                                                     = mrOK then
        begin
             SaveForUndo;
             FileOpened := False;
             MyTextFile.Clear;
             WinPadForm.Caption := 'WinPad';
             Application.Title := 'WinPad';
        end;
end;

procedure TWinPadForm.OpenBtnClick(Sender: TObject);
{ Open an existing file by loading from disc }
begin
     if not MyTextFile.Modified and not FileOpened then
     { If a file is not open or been modified, open a new file }
     begin
          if OpenDialog.Execute then
          begin
               SaveForUndo;
               OpenFile;
               SaveForUndo;
          end;
     end
     else
         { Otherwise confirm whether the current file is to be closed }
         if MessageDlg('Close Current File?',mtInformation,[mbOK,mbCancel],0) =
                                                                       mrOK then
            if OpenDialog.Execute then
            begin
                 SaveForUndo;
                 OpenFile;
                 SaveForUndo;
            end;
end;

procedure TWinPadForm.SaveBtnClick(Sender: TObject);
{ If the file has a pathname, save it under that name, else open a save dialog
{ and prompt the user for a full file & path name before saving the text file}
begin
     if FileOpened then
        MyTextFile.Lines.SaveToFile(OpenDialog.Filename)
     else
         SaveAsItemClick(Self);
end;

procedure TWinPadForm.SaveAsItemClick(Sender: TObject);
{ Open a save dialog and prompt the user for a full file & path before saving
{ the text file }
begin
     if SaveDialog.Execute then
     begin
          MyTextFile.Lines.SaveToFile(SaveDialog.Filename);
          OpenDialog.Filename := SaveDialog.FileName;
          FileOpened := True;
          WinPadForm.Caption := 'WinPad - '+ ansilowercase(OpenDialog.Filename);
          Application.Title := 'WinPad - ' + ansilowercase(OpenDialog.Filename);
     end;
end;

procedure TWinPadForm.PrintBtnClick(Sender: TObject);
{ Open a print dialog & print the currently opened file }
var
   Line     : Integer;
   Printext : System.Text;

begin
     if PrintDialog.Execute then
     begin
          AssignPrn(Printext);
          ReWrite(Printext);
          Printer.Canvas.Font := MyTextFile.Font;
          for Line := 0 to MyTextFile.Lines.Count - 1 do
              WriteLn(Printext, MyTextFile.Lines[Line]);
          System.Close(Printext);
     end;
end;

procedure TWinPadForm.PrintSetupClick(Sender: TObject);
{ Open the printer setup dialog }
begin
     PrinterSetupDialog.Execute;
end;

procedure TWinPadForm.CutBtnClick(Sender: TObject);
{ Cut selected text from the text file memo box to the clipboard }
begin
     SaveForUndo;
     MyTextFile.CutToClipboard;
end;

procedure TWinPadForm.CopyBtnClick(Sender: TObject);
{ Copy selected text from the text file memo box to the clipboard }
begin
     SaveForUndo;
     MyTextFile.CopyToClipboard;
end;

procedure TWinPadForm.PasteBtnClick(Sender: TObject);
{ Paste selected text from the clipboard to the text file memo box }
begin
     SaveForUndo;
     MyTextFile.PasteFromClipBoard;
end;

procedure TWinPadForm.DeleteBtnClick(Sender: TObject);
{ Clear selected text from the text file memo box }
begin
     SaveForUndo;
     MyTextFile.ClearSelection;
end;

procedure TWinPadForm.SelectAllBtnClick(Sender: TObject);
{ Select all text in the text file memo box }
begin
     MyTextFile.SelectAll;
end;

procedure TWinPadForm.HelpBtnClick(Sender: TObject);
{ Display the WinPad product information form as a modal dialog }
begin
     AboutBox.ShowModal;
end;

procedure TWinPadForm.TimeDateItemClick(Sender: TObject);
{ Insert the date and time into the document at the current caret position }
begin
     ClipBoard.AsText := ' ' + DateToStr(Now) + ' ' + TimeToStr(Now) + ' ';
     PasteBtnClick(Self); { This will also save undo information }
     Clipboard.Clear;
end;

procedure TWinPadForm.WordWrapItemClick(Sender: TObject);
{ Looking at the text file memo box, word wrap is toggled on or off and the
{ scrollbars adjusted accordingly depending on the current state of the toggle }
begin
     with MyTextFile do
     begin
          WordWrap := not WordWrap;
          if WordWrap then
             Scrollbars := ssVertical
          else
              Scrollbars := ssBoth;

          WordWrapItem.Checked := WordWrap;
     end;
end;

procedure TWinPadForm.FindBtnClick(Sender: TObject);
{ Open the find dialog }
begin
     FindDialog.Execute;
     FindNextItem.Enabled := True;
end;

procedure TWinPadForm.FindNextItemClick(Sender: TObject);
{ Find the next occurence of the word provided by the find dialog }
begin
     Find(FindDialog);
end;

procedure TWinPadForm.Find(Sender: TObject);
{ Use the SearchMemo procedure to locate the word or display an error}
begin
     with Sender as TFindDialog do
          if not SearchMemo(MyTextFile, FindText, Options) then
              MessageDlg('Cannot Find ' + FindText, mtInformation, [mbOk], 0);
end;

procedure TWinPadForm.ReplaceBtnClick(Sender: TObject);
{ Open the find and replace dialog }
begin
     ReplaceDialog.Execute;
end;

procedure TWinPadForm.Replace(Sender: TObject);
{ Replace the text specified by the user in the Replace dialog }
var
   Found: Boolean;

begin
     UndoMemo.Clear;
     UndoMemo.WordWrap := MyTextFile.WordWrap;
     UndoMemo.Lines    := MyTextFile.Lines;
     UndoMemo.Font     := MyTextFile.Font;
     with ReplaceDialog do
     begin
          if AnsiCompareText(MyTextFile.SelText, FindText) = 0 then
             MyTextFile.SelText := ReplaceText;

          Found := SearchMemo(MyTextFile, FindText, Options);

          while Found and (frReplaceAll in Options) do
          begin
               MyTextFile.SelText := ReplaceText;
               Found := SearchMemo(MyTextFile, FindText, Options);
          end;

          if (not Found) and (frReplace in Options) then
             MessageDlg('Cannot Find ' + FindText, mtInformation, [mbOk], 0);
     end;
end;

procedure TWinPadForm.FontItemClick(Sender: TObject);
{ Displays a font dialog and allows changes to the text file memo box's font }
begin
     SaveForUndo;
     if FontDialog.Execute then
        MyTextFile.Font := FontDialog.Font;
end;

procedure TWinPadForm.UndoBtnClick(Sender: TObject);
{ Reverse the last editing procedure }
begin
     with MyTextFile do
     begin
          Clear;
          WordWrap := UndoMemo.WordWrap;
          Font     := UndoMemo.Font;
          Lines    := UndoMemo.Lines;
     end;
end;

procedure TWinPadForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
{ Check whether the current file needs to be saved }
begin
     if FileOpened or MyTextFile.Modified then
        if MessageDlg('Do you wish to save this file?',mtInformation,
                                                    [mbYes,mbNo],0) = mrYes then
        begin
             SaveBtnClick(Self);
             CanClose := True;
        end
     else
         CanClose := True;
end;

procedure TWinPadForm.FormActivate(Sender: TObject);
{ Show product information }
begin
     AboutBox.ShowModal;
end;

procedure TWinPadForm.FormResize(Sender: TObject);
{ Prevent the status and toolbar from being hidden }
begin
     if width < 442 then
        width := 442;

     if height < 145 then
        height := 145;
end;

end.
